@@ -302,6 +302,23 @@ ensure_bus_data (DBusConnection *connection)
   return bd;
 }
 
+/* internal function that checks to see if this
+   is a shared bus connection and if it is unref it */
+void
+_dbus_bus_check_connection_and_unref (DBusConnection *connection)
+{
+  if (bus_connections[DBUS_BUS_SYSTEM] == connection)
+    {
+      bus_connections[DBUS_BUS_SYSTEM] = NULL;
+      dbus_connection_unref (connection);
+    }
+  else if (bus_connections[DBUS_BUS_SESSION] == connection)
+    {
+      bus_connections[DBUS_BUS_SESSION] = NULL;
+      dbus_connection_unref (connection);
+    }
+}
+
 static DBusConnection *
 internal_bus_get (DBusBusType  type,
 	      DBusError   *error, dbus_bool_t private)
@@ -385,7 +402,11 @@ internal_bus_get (DBusBusType  type,
     }
 
   if (!private)
-    bus_connections[type] = connection;
+    {
+      /* get a hard ref to the connection */
+      bus_connections[type] = connection;
+      dbus_connection_ref (bus_connections[type]);
+    }
   
   bd = ensure_bus_data (connection);
   _dbus_assert (bd != NULL);
@@ -409,8 +430,6 @@ internal_bus_get (DBusBusType  type,
  * connection to the bus already exists, then that connection is
  * returned.  Caller owns a reference to the bus.
  *
- * @todo alex thinks we should nullify the connection when we get a disconnect-message.
- *
  * @param type bus type
  * @param error address where an error can be returned.
  * @returns a DBusConnection with new ref
@@ -701,10 +720,6 @@ dbus_bus_get_unix_user (DBusConnection *connection,
  * (last-started-wins) or be queued up (first-started-wins) according
  * to whether #DBUS_NAME_FLAG_ALLOW_REPLACEMENT was given.
  * 
- * @todo this all seems sort of broken. Shouldn't the flags be a property
- * of the name, not the app requesting the name? What are the use-cases
- * other than the "text editor" thing and how are we supporting them?
- * 
  * @param connection the connection
  * @param name the name to request
  * @param flags flags
