@@ -99,8 +99,6 @@ typedef unsigned long dbus_gid_t;
 #define DBUS_UID_FORMAT "%lu"
 #define DBUS_GID_FORMAT "%lu"
 
-#define DBUS_CONSOLE_DIR "/var/run/console/"
-
 /**
  * Struct representing socket credentials
  */
@@ -255,11 +253,15 @@ dbus_bool_t  _dbus_directory_get_next_file (DBusDirIter      *iter,
                                             DBusError        *error);
 void         _dbus_directory_close         (DBusDirIter      *iter);
 
+dbus_bool_t  _dbus_check_dir_is_private_to_user    (DBusString *dir,
+                                                    DBusError *error);
 
-dbus_bool_t _dbus_generate_random_bytes (DBusString *str,
-                                         int         n_bytes);
-dbus_bool_t _dbus_generate_random_ascii (DBusString *str,
-                                         int         n_bytes);
+void        _dbus_generate_random_bytes_buffer (char       *buffer,
+                                                int         n_bytes);
+dbus_bool_t _dbus_generate_random_bytes        (DBusString *str,
+                                                int         n_bytes);
+dbus_bool_t _dbus_generate_random_ascii        (DBusString *str,
+                                                int         n_bytes);
 
 const char *_dbus_errno_to_string  (int errnum);
 const char* _dbus_error_from_errno (int error_number);
@@ -345,6 +347,9 @@ dbus_bool_t _dbus_user_at_console (const char *username,
       _DBUS_BYTE_OF_PRIMITIVE (a, 6) == _DBUS_BYTE_OF_PRIMITIVE (b, 6) &&       \
       _DBUS_BYTE_OF_PRIMITIVE (a, 7) == _DBUS_BYTE_OF_PRIMITIVE (b, 7))
 
+dbus_bool_t _dbus_parse_uid (const DBusString  *uid_str,
+                             dbus_uid_t        *uid);
+
 DBUS_END_DECLS
 
 #endif /* DBUS_SYSDEPS_H */
