@@ -28,3 +28,6 @@ if test -z "$DBUS_TEST_NAME_IN_RUN_TEST"; then
 fi 
 echo "running test-names"
 libtool --mode=execute $DEBUG $DBUS_TOP_BUILDDIR/test/name-test/test-names || die "test-client failed"
+
+echo "running test-pending-call-dispatch"
+libtool --mode=execute $DEBUG $DBUS_TOP_BUILDDIR/test/name-test/test-pending-call-dispatch || die "test-client failed"
