@@ -884,17 +884,17 @@ main (int argc, char **argv)
       else
 	{
 	  if (c_shell_syntax)
-	    printf ("setenv DBUS_SESSION_BUS_ADDRESS '%s'\n", bus_address);	
+	    printf ("setenv DBUS_SESSION_BUS_ADDRESS '%s';\n", bus_address);	
 	  else
 	    {
-	      printf ("DBUS_SESSION_BUS_ADDRESS='%s'\n", bus_address);
+	      printf ("DBUS_SESSION_BUS_ADDRESS='%s';\n", bus_address);
 	      if (bourne_shell_syntax)
-		printf ("export DBUS_SESSION_BUS_ADDRESS\n");
+		printf ("export DBUS_SESSION_BUS_ADDRESS;\n");
 	    }
 	  if (c_shell_syntax)
-	    printf ("set DBUS_SESSION_BUS_PID=%ld\n", (long) bus_pid);
+	    printf ("set DBUS_SESSION_BUS_PID=%ld;\n", (long) bus_pid);
 	  else
-	    printf ("DBUS_SESSION_BUS_PID=%ld\n", (long) bus_pid);
+	    printf ("DBUS_SESSION_BUS_PID=%ld;\n", (long) bus_pid);
 	}
 	  
       verbose ("dbus-launch exiting\n");
